/*
 *  App.cpp
 *  Created by Seth Robinson on 3/6/09.
 *  For license info, check the license.txt file that should have come with this.
 *
 */

#include "App.h"
#include "MessageManager.h"

//create the classes that our globally library expects to exist somewhere.

MessageManager g_messageManager;
MessageManager * GetMessageManager() {return &g_messageManager;}


App g_App;
BaseApp * GetBaseApp() {return &g_App;}

App::App()
{}

App::~App()
{
}

bool App::Init()
{
	if (m_bInitted)	
	{
		return true;
	}
	
	if (!BaseApp::Init()) return false;
	
	glDisable(GL_CULL_FACE); //so we can see the back of the triangle too

	LogMsg("Save path is %s", GetSavePath().c_str());

	//GetBaseApp()->SetFPSVisible(true);

	return true;
}

void App::Kill()
{
	BaseApp::Kill();
}

void App::Draw()
{

	glClearColor(0, 0, 0, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
	if (GetOrientation() != ORIENTATION_PORTRAIT)
	{
		glRotatef(-90.0f, 0.0f, 0.0f, 1.0f);
	}

	glRotatef(float( (GetGameTick()/10) %360) , 0, 1, 0); //rotate it
	RenderGLTriangle();
	glPopMatrix();

	//draw our game stuff


	//the base handles actually drawing the GUI stuff over everything else
	BaseApp::Draw();
}

void App::Update()
{
	
	BaseApp::Update();
}

void App::OnScreenSizeChange()
{
	BaseApp::OnScreenSizeChange();
}

void App::OnMessage(Message &m)
{
	switch (m.GetType())
	{
		case MESSAGE_TYPE_GUI_CLICK_START:
			LogMsg("Click started at %d, %d", m.GetParm1(), m.GetParm2());
			break;
		case MESSAGE_TYPE_GUI_CLICK_END:
			LogMsg("Click ended %d, %d", m.GetParm1(), m.GetParm2());
			break;
		case MESSAGE_TYPE_GUI_CLICK_MOVE:
			LogMsg("Click moved to %d, %d", m.GetParm1(), m.GetParm2());
			break;
			
	}
	
	BaseApp::OnMessage(m);
}

App * GetApp() 
{
	return &g_App;
}
