#include "GameTimer.h"

#define C_MAXIMUM_DELTA_ALLOWED 100 //phones aren't allowed to be slower than this

GameTimer::GameTimer()
{
	Reset();
}

GameTimer::~GameTimer()
{
}

void GameTimer::Update()
{
	//compute delta time
	m_timeMS = GetSystemTimeTick();
	m_deltaMS =  m_timeMS - m_lastTimeMS;
	m_lastTimeMS =  m_timeMS;

	//we have a maximum delta because above a certain point all collision detection would
	//be broken. 
	if (m_deltaMS > C_MAXIMUM_DELTA_ALLOWED) m_deltaMS = C_MAXIMUM_DELTA_ALLOWED;

	if (!m_bGameTimerPaused)
	{
		//advance game timer 
		m_gameTimer += m_deltaMS;
	}
	if (m_deltaMS < 1) 
	{
		m_deltaMS = 1; //avoid divide by zero errors later
	}
	
	m_deltaFloat = float(m_deltaMS)/ (1000.0f/50.0f); //plan on 50 FPS being about average, which will return 1.0

	if (m_fpsTimer < m_timeMS)
	{
		m_fpsTimer = m_timeMS+1000;
		m_fps = m_fpsTemp;
		m_fpsTemp = 0;
	}

	m_fpsTemp++;
}

void GameTimer::Reset()
{
	m_lastTimeMS = m_timeMS = GetSystemTimeTick(); //make sure this is valid now
	m_bGameTimerPaused = false;
	m_fps = m_fpsTemp = 0;
	m_fpsTimer = 0;
	m_deltaFloat = 1;
}
