#include "MessageManager.h"
#include "BaseApp.h"


void Message::SetDeliveryTime( int deliveryTimeMS )
{
	if (m_timerMethod == MESSAGE_TIMER_GAME)
	{
		m_deliveryTime = deliveryTimeMS + GetBaseApp()->GetGameTick();
	} else m_deliveryTime = deliveryTimeMS + GetBaseApp()->GetTick();
}

MessageManager::MessageManager()
{
}

MessageManager::~MessageManager()
{
}

void MessageManager::AddGUI( eMessageType type, int part1, int part2 )
{
	Message m(MESSAGE_CLASS_GUI, MESSAGE_TIMER_SYSTEM, type);
	m.SetParm1(part1);
	m.SetParm2(part2);
	Send(m);
}

void MessageManager::AddGUI( eMessageType type, int part1, int part2,  int deliverTimeMS)
{
	Message m(MESSAGE_CLASS_GUI, MESSAGE_TIMER_SYSTEM, type);
	m.SetParm1(part1);
	m.SetParm2(part2);
	m.SetDeliveryTime(deliverTimeMS);
	Send(m);
}

void MessageManager::AddMessageToList(list <Message> &messageList, Message &m)
{
	//insert sorted by delivery time
	list <Message>::reverse_iterator itor = messageList.rbegin();
	while (itor != messageList.rend())
	{
		if (itor->GetDeliveryTime() > m.GetDeliveryTime())
		{
			itor++;
		} else
		{
			break;
		}
	}

	//add it
	messageList.insert(itor.base(), m);
}

void MessageManager::Send(Message &m)
{
	if (m.GetTimingMethod() == MESSAGE_TIMER_GAME)
	{
		AddMessageToList(m_gameMessages, m);
	} else
	{
		AddMessageToList(m_systemMessages, m);
	}
}


void MessageManager::DumpMessagesInList(list<Message> m)
{
	list<Message>::iterator itor = m.begin();

	string s;

	while (itor != m.end())
	{
		s.clear();
		if (itor->GetTimingMethod() == MESSAGE_TIMER_GAME)
		{
			s += "Game: ";
		} else
		{
			s += "System: ";
		}

		s += " Delivery: " + toString(itor->GetDeliveryTime());
		s += " Type: " + toString(itor->GetType());
		s += " Parm1: " + toString(itor->GetParm1());
		s += " Parm2: " + toString(itor->GetParm2());

		LogMsg(s.c_str());
		itor++;
	}
}

//thing used for debugging
void MessageManager::DumpMessages()
{
	LogMsg("Dumping system messages...");
	DumpMessagesInList(m_gameMessages);
	LogMsg("Dumping game messages...");
	DumpMessagesInList(m_systemMessages);

}


void MessageManager::Deliver(Message &m)
{
	GetBaseApp()->OnMessage(m);
}

void MessageManager::Update()
{

	while (!m_systemMessages.empty() && m_systemMessages.begin()->GetDeliveryTime() <= GetBaseApp()->GetTick())
	{
		Message m = *m_systemMessages.begin();
		m_systemMessages.pop_front();
		Deliver(m);
	}


	while (!m_gameMessages.empty() && m_gameMessages.begin()->GetDeliveryTime() <= GetBaseApp()->GetGameTick())
	{
		Message m = *m_gameMessages.begin();
		m_gameMessages.pop_front();
		Deliver(m);
	}

}