//  ***************************************************************
//  MessageManager - Creation date: 03/18/2009
//  -------------------------------------------------------------
//  Robinson Technologies Copyright (C) 2009 - All Rights Reserved
//
//  ***************************************************************
//  Programmer(s):  Seth A. Robinson (seth@rtsoft.com)
//  ***************************************************************

#ifndef MessageManager_h__
#define MessageManager_h__

#include "PlatformSetup.h"


enum eMessageClass
{
	MESSAGE_CLASS_GAME,
	MESSAGE_CLASS_GUI
};

enum eMessageTimer
{
	MESSAGE_TIMER_SYSTEM,
	MESSAGE_TIMER_GAME //message deliveries are paused when the game is paused/slowed down
};

enum eMessageType
{
	MESSAGE_TYPE_GUI_CLICK_START,
	MESSAGE_TYPE_GUI_CLICK_END,
	MESSAGE_TYPE_GUI_CLICK_MOVE
};

class Message
{

	public:

	Message(eMessageClass messageClass, eMessageTimer timer, eMessageType type) : m_class(messageClass), m_timerMethod(timer), m_type(type)
	{
		m_deliveryTime = 0; //delivery right away
	}

	void SetParm1(int parm1) {m_parm1 = parm1;}
	int GetParm1() {return m_parm1;}
	void SetParm2(int parm2) {m_parm2 = parm2;}
	int GetParm2() {return m_parm2;}
	eMessageTimer GetTimingMethod() {return m_timerMethod;}
	unsigned int GetDeliveryTime() {return m_deliveryTime;}
	eMessageType GetType() {return m_type;}
	void SetDeliveryTime(int deliveryTimeMS);

private:

	eMessageType m_type;
	eMessageTimer m_timerMethod;
	eMessageClass m_class;
	int m_parm1;
	int m_parm2;
	unsigned int m_deliveryTime;

};

class MessageManager
{
public:
	MessageManager();
	virtual ~MessageManager();

	void AddGUI(eMessageType type, int part1, int part2);
	void AddGUI( eMessageType type, int part1, int part2, int deliverTimeMS);
	void DumpMessages();
	void Update(); //run every tick

	
private:

	void Send(Message &m);
	void AddMessageToList(list <Message> &messageList, Message &m);
	void DumpMessagesInList(list<Message> m);
	void Deliver(Message &m);
	//a separate queue for each timing system
	list <Message> m_gameMessages;
	list <Message> m_systemMessages;

};

#endif // MessageManager_h__