/*
 *  RenderUtils.cpp
 *  
 *
 *  Created by Seth Robinson on 3/18/09.
 *  Copyright 2009 Robinson Technologies. All rights reserved.
 *
 */

#include "RenderUtils.h"
#include "BaseApp.h"


//defaults
int g_screenSizeY = 480;
int g_screenSizeX = 320;
int g_orientation = ORIENTATION_PORTRAIT;


void RenderGLTriangle()
{
	//let's draw a simple triangle 
	/*
	0
	1 2
	*/

	const float triSize = 0.4f;

	GLfloat vVertices[] = {0.0f, triSize, 0.0f,
		-triSize, -triSize, 0.0f,
		triSize, -triSize, 0.0f};

	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FLOAT,  0, vVertices);
	glDisable( GL_BLEND );

	unsigned int rgba = MAKE_RGBA(255,255,255,255);
	glColor4x( (rgba >>8 & 0xFF)*256,  (rgba>>16& 0xFF)*256, (rgba>>24& 0xFF)*256, (rgba&0xFF)*256);
	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
	glDisableClientState(GL_COLOR_ARRAY);
	glDrawArrays(GL_TRIANGLES, 0, 3);
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	glEnable( GL_BLEND );
}

void orthographic()		   
{                   
	glMatrixMode(GL_PROJECTION);		   
	glLoadIdentity();					   
	glOrthof( 0, float(GetScreenSizeX()), float(GetScreenSizeY()), 0, 1, 0 );				
	glMatrixMode(GL_MODELVIEW);				
	glLoadIdentity();					
}

int GetScreenSizeY()
{
	return g_screenSizeY;
}

int GetScreenSizeX()
{
	return g_screenSizeX;
}

int GetOrientation() {return g_orientation;}

void SetupScreenInfo(int x, int y, int orientation)
{
		
		if (g_screenSizeX == x && g_screenSizeY == y && g_orientation == orientation)
		{
			return;
		}
		LogMsg("Setting screen info to %d, %d, mode %d", x, y, orientation);

		g_screenSizeX = x;
		g_screenSizeY = y;
		g_orientation = orientation;
		
		GetBaseApp()->OnScreenSizeChange();
}

void ConvertCoordinatesIfRequired(int &xPos, int &yPos)
{
	if (g_orientation == ORIENTATION_LANDSCAPE)
	{
		swap(xPos, yPos);
		yPos = GetScreenSizeY()-yPos;
	} else
	{
		//no conversion required
	}
}

void ConvertCoordinatesIfRequired(float &xPos, float &yPos)
{
	if (g_orientation == ORIENTATION_LANDSCAPE)
	{
		swap(xPos, yPos);
		yPos = GetScreenSizeY()-yPos;
	} else
	{
		//no conversion required
	}
}

bool g_needsOrthoModeSet = true;

bool NeedsOrthoSet() 
{
	return g_needsOrthoModeSet;
}

void ResetOrthoFlag()
{
	if (!NeedsOrthoSet())
	{
		glDepthMask(GL_TRUE);
		glEnable(GL_DEPTH_TEST);   
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
	}
	g_needsOrthoModeSet = true;
}

void SetOrthoMode()
{
	g_needsOrthoModeSet = false;
}

bool InLandscapeGUIMode()
{
	return GetOrientation() == 3 || GetScreenSizeX() >= 480;
}